Attribute VB_Name = "ModLibUSB0dll"
Option Explicit
'Some information about the Module:
'----------------------------------
'Functions exported by libusb0.dll:
'    usb_init
'    usb_find_busses
'    usb_find_devices
'    usb_get_version
'
'    usb_bulk_read
'    usb_bulk_setup_async
'    usb_bulk_write
'    usb_cancel_async
'    usb_claim_interface
'    usb_clear_halt
'    usb_close
'    usb_control_msg
'    usb_device
'    usb_free_async
'    usb_get_busses
'    usb_get_descriptor
'    usb_get_descriptor_by_endpoint
'    usb_get_string
'    usb_get_string_simple
'    usb_install_driver_np
'    usb_install_driver_np_rundll
'    usb_install_needs_restart_np
'    usb_install_service_np
'    usb_install_service_np_rundll
'    usb_interrupt_read
'    usb_interrupt_setup_async
'    usb_interrupt_write
'    usb_isochronous_setup_async
'    usb_open
'    usb_reap_async
'    usb_reap_async_nocancel
'    usb_release_interface
'    usb_reset
'    usb_resetep
'    usb_set_altinterface
'    usb_set_configuration
'    usb_set_debug
'    usb_strerror
'    usb_submit_async
'    usb_touch_inf_file_np
'    usb_touch_inf_file_np_rundll
'    usb_uninstall_service_np
'    usb_uninstall_service_np_rundll

'*****************************************************************
'**** All the Constants...
'*****************************************************************
Public Const LIBUSB_PATH_MAX  As Long = 512
'Device and/or Interface Class codes
Public Const USB_CLASS_PER_INTERFACE As Long = 0  '/* for DeviceClass */
Public Const USB_CLASS_AUDIO         As Long = 1
Public Const USB_CLASS_COMM          As Long = 2
Public Const USB_CLASS_HID           As Long = 3
Public Const USB_CLASS_PRINTER       As Long = 7
Public Const USB_CLASS_MASS_STORAGE  As Long = 8
Public Const USB_CLASS_HUB           As Long = 9
Public Const USB_CLASS_DATA          As Long = 10
Public Const USB_CLASS_VENDOR_SPEC   As Long = &HFF

'Descriptor types
Public Const USB_DT_DEVICE    As Long = &H1
Public Const USB_DT_CONFIG    As Long = &H2
Public Const USB_DT_STRING    As Long = &H3
Public Const USB_DT_INTERFACE As Long = &H4
Public Const USB_DT_ENDPOINT  As Long = &H5

Public Const USB_DT_HID       As Long = &H21
Public Const USB_DT_REPORT    As Long = &H22
Public Const USB_DT_PHYSICAL  As Long = &H23
Public Const USB_DT_HUB       As Long = &H29

'Descriptor sizes per descriptor type
Public Const USB_DT_DEVICE_SIZE            As Long = 18
Public Const USB_DT_CONFIG_SIZE            As Long = 9
Public Const USB_DT_INTERFACE_SIZE         As Long = 9
Public Const USB_DT_ENDPOINT_SIZE          As Long = 7
Public Const USB_DT_ENDPOINT_AUDIO_SIZE    As Long = 9 '/* Audio extension */
Public Const USB_DT_HUB_NONVAR_SIZE        As Long = 7

'Enpoint Descriptors...
Public Const USB_MAXENDPOINTS              As Long = 32
Public Const USB_ENDPOINT_ADDRESS_MASK     As Long = &HF '/* in bEndpointAddress */
Public Const USB_ENDPOINT_DIR_MASK         As Long = &H80
Public Const USB_ENDPOINT_TYPE_MASK        As Long = &H3 '/* in bmAttributes */
Public Const USB_ENDPOINT_TYPE_CONTROL     As Long = 0
Public Const USB_ENDPOINT_TYPE_ISOCHRONOUS As Long = 1
Public Const USB_ENDPOINT_TYPE_BULK        As Long = 2
Public Const USB_ENDPOINT_TYPE_INTERRUPT   As Long = 3

'Diverse
Public Const USB_MAXINTERFACES As Long = 32
Public Const USB_MAXALTSETTING As Long = 128
Public Const USB_MAXCONFIG     As Long = 8

'Standard requests
Public Const USB_REQ_GET_STATUS        As Long = &H0
Public Const USB_REQ_CLEAR_FEATURE     As Long = &H1
'/* 0x02 is reserved */
Public Const USB_REQ_SET_FEATURE       As Long = &H3
'/* 0x04 is reserved */
Public Const USB_REQ_SET_ADDRESS       As Long = &H5
Public Const USB_REQ_GET_DESCRIPTOR    As Long = &H6
Public Const USB_REQ_SET_DESCRIPTOR    As Long = &H7
Public Const USB_REQ_GET_CONFIGURATION As Long = &H8
Public Const USB_REQ_SET_CONFIGURATION As Long = &H9
Public Const USB_REQ_GET_INTERFACE     As Long = &HA
Public Const USB_REQ_SET_INTERFACE     As Long = &HB
Public Const USB_REQ_SYNCH_FRAME       As Long = &HC

Public Const USB_TYPE_STANDARD   As Long = &H0&  '    = (0x00 << 5) '0 ShiftLeft 5 is still 0
Public Const USB_TYPE_CLASS      As Long = &H20& '    = (0x01 << 5) '1 ShiftLeft 5 must be 1 * 2^5 btw: 2^5 aka 32 (decimal) or &H20
Public Const USB_TYPE_VENDOR     As Long = &H40& '    = (0x02 << 5) '2 ShiftLeft 5 must be 2 * 2^5
Public Const USB_TYPE_RESERVED   As Long = &H60& '    = (0x03 << 5) '3 ShiftLeft 5 must be 3 * 2^5

Public Const USB_RECIP_DEVICE    As Long = &H0&
Public Const USB_RECIP_INTERFACE As Long = &H1&
Public Const USB_RECIP_ENDPOINT  As Long = &H2&
Public Const USB_RECIP_OTHER     As Long = &H3&

'Various libusb API related stuff
Public Const USB_ENDPOINT_IN     As Long = &H80
Public Const USB_ENDPOINT_OUT    As Long = &H0

'Error codes
Public Const USB_ERROR_BEGIN     As Long = 500000

'----------------------------------
'Types in this Module:
'    USB_Bus
'    USB_Device
'    USB_Device_Descriptor
'    USB_Config_Descriptor
'    USB_Interface
'    USB_Interface_Descriptor
'    USB_EndPoint_Descriptor
'
'    PS.! There is one structure/type missing, that is
'         the USB_Dev_Handle:
'
'         struct usb_dev_handle;
'         typedef struct usb_dev_handle usb_dev_handle;
'
'         I just have no clue on how to convert the above structure...
'
' Oliver Meyer (OM): i assume this must simply be Long
' or use this Trick instead:
Public Enum usb_dev_handle
    [_]
End Enum

'****************************************************************
'**** Here are all the structures:
'****************************************************************
'----------------------------------
'   This module is written by Richard Hagen
'   Thx to all the people in LibUSB & LibUSB-Win32 teams &
'   the people that help them...
'
'   You can modify or rewrite this.... I dont cear, just
'   remember, if you rewite it and its better that this one,
'   Please send a copy to me;) @ richard.hagen@gmail.com... thx
'
'/* All standard descriptors have these 2 fields in common */
Public Type usb_descriptor_header
  bLength             As Byte '1 'unsigned char
  bDescriptorType     As Byte '1
End Type                 'SUM: 2

'/* String descriptor */
Public Type usb_string_descriptor
  bLength             As Byte    '1 'unsigned char
  bDescriptorType     As Byte    '1
  wData(1 To 1)       As Integer '2 'unsigned short '????
End Type                    'SUM: 4

'/* HID descriptor */
Public Type usb_hid_descriptor
  bLength             As Byte    '1
  bDescriptorType     As Byte    '1
  bcdHID              As Integer '2 '  unsigned short
  bCountryCode        As Byte    '1
  bNumDescriptors     As Byte    '1
End Type                    'SUM: 6

'/* Endpoint descriptor */
Public Type usb_endpoint_descriptor
  bLength             As Byte    '1 'unsigned char
  bDescriptorType     As Byte    '1
  bEndpointAddress    As Byte    '1
  bmAttributes        As Byte    '1
  wMaxPacketSize      As Integer '2 'unsigned short
  bInterval           As Byte    '1
  bRefresh            As Byte    '1
  bSynchAddress       As Byte    '1
  pExtra              As Long    '4 'unsigned char *extra; /* Extra descriptors */ 'oder As String?
  extralen            As Long    '4
End Type                   'SUM: 17 'really ??? why so bescheuert?

'/* Interface descriptor */
Public Type usb_interface_descriptor
  bLength             As Byte    '1 'unsigned char
  bDescriptorType     As Byte    '1
  bInterfaceNumber    As Byte    '1
  bAlternateSetting   As Byte    '1
  bNumEndpoints       As Byte    '1
  bInterfaceClass     As Byte    '1
  bInterfaceSubClass  As Byte    '1
  bInterfaceProtocol  As Byte    '1
  iInterface          As Byte    '1
  pEndpoint           As Long    '4 'struct usb_endpoint_descriptor *endpoint
  pExtra              As Long    '4 'unsigned char *extra; /* Extra descriptors */
  extralen            As Long    '4
End Type                   'SUM: 21

Public Type usb_interface
  pUsb_interface_descriptor As Long '4 ' pointer to struct usb_interface_descriptor *altsetting;
  num_altsetting            As Long '4
End Type                       'SUM: 8

'/* Configuration descriptor information.. */
'Tja hier knnte das packed probleme bereiten
Public Type usb_config_descriptor
  bLength             As Byte    '1'unsigned char
  bDescriptorType     As Byte    '1
  wTotalLength        As Integer '2 'unsigned short
  bNumInterfaces      As Byte    '1
  bConfigurationValue As Byte    '1
  iConfiguration      As Byte    '1
  bmAttributes        As Byte    '1
  MaxPower            As Byte    '1
  pUsb_interface      As Long    '4 ' pointer to a struct usb_interface *interface;
  pExtra              As Long    '4 ' pointer to unsigned char *extra; '/* Extra descriptors */
  extralen            As Long    '4
End Type                    'SUM: 21 '??? oh shit why so bescheuert?

'/* Device descriptor */
Public Type usb_device_descriptor
  bLength             As Byte    '1   'unsigned char
  bDescriptorType     As Byte    '1
  bcdUSB              As Integer '2 'unsigned short
  bDeviceClass        As Byte    '1
  bDeviceSubClass     As Byte    '1
  bDeviceProtocol     As Byte    '1
  bMaxPacketSize0     As Byte    '1
  idVendor            As Integer '2
  idProduct           As Integer '2
  bcdDevice           As Integer '2
  iManufacturer       As Byte    '1
  iProduct            As Byte    '1
  iSerialNumber       As Byte    '1
  bNumConfigurations  As Byte    '1
End Type                'SUM:    18 'geht das? das kann Probleme machen

Public Type usb_ctrl_setup
  bRequestType        As Byte    '1
  bRequest            As Byte    '1
  wValue              As Integer '2 'unsigned short
  wIndex              As Integer '2
  wLength             As Integer '2
End Type                    'SUM: 8

'/* Data types */
'/* struct usb_device; */
'/* struct usb_bus; */
Public Type usb_device
  next                As Long  '4 'pointer to the next USB_Device
  prev                As Long  '4 'pointer to the previous USB_Device
  filename(0 To LIBUSB_PATH_MAX - 1) As Byte '512
  Bus                 As Long  '4 'pointer to the parent USB_Bus
  Descriptor          As usb_device_descriptor '18 'here it is not a pointer to a usb_device_Descriptor
  Config              As Long  '4 'pointer to a USB_Config_Descriptor
  pDev                As Long  '4 pointer to void  'Darwin support ???
  devnum              As Byte  '1
  num_children        As Byte  '1
  ppChildren          As Long  '4 'pointer to a pointer to a usb_device
End Type              'SUM:   556

Public Type usb_bus
  next                As Long      '4 'pointer to the next usb_bus
  prev                As Long      '4 'pointer to the previous usb_bus
  dirname(0 To LIBUSB_PATH_MAX - 1) As Byte '512
  devices             As Long      '4 'pointer to the first usb_device
  location            As Long      '4 'Int64
  pRoot_dev           As Long      '4 'pointer to a usb_device
End Type                    'SUM: 532
'Anmerkung: im Delphi-Quellcode fehlt hier das pRoot_dev deswegen liefert sizeof in delphi hier 528 !

'/* Version information, Windows specific */
Public Type usb_version_
    major  As Long '4
    minor  As Long '4
    micro  As Long '4
    nano   As Long '4
End Type     'SUM: 16
Public Type usb_version
    dll    As usb_version_ '16
    driver As usb_version_ '16
End Type              'SUM: 32
'
'***************************************************************
'**** The API declarations...
'***************************************************************
'  /* Function prototypes */

'  /* usb.c */
'Public Declare Function usb_open Lib "libusb0" _
'    (ByRef dev As usb_device) As usb_dev_handle
Public Declare Function usb_open Lib "libusb0" _
    (ByVal pDev As Long) As usb_dev_handle 'pDev = pointer to a usb_device-struct
Public Declare Function usb_close Lib "libusb0" _
    (ByRef dev As usb_dev_handle) As Long
Public Declare Function usb_get_string Lib "libusb0" _
    (ByRef dev As usb_dev_handle, ByVal Index As Long, ByVal LangId As Long, ByRef Buf As String, ByVal BufLen As Long) As Long

'  /* descriptors.c */
Public Declare Function usb_get_descriptor_by_endpoint Lib "libusb0" _
    (ByRef udev As usb_dev_handle, ByVal ep As Long, ByVal btype As Byte, ByVal Index As Byte, ByRef pbuf As Long, ByVal size As Long) As Long
Public Declare Function usb_get_descriptor Lib "libusb0" _
    (ByRef udev As usb_dev_handle, ByVal btype As Byte, ByVal Index As Byte, ByRef pbuf As Long, size As Long) As Long

'  /* <arch>.c */
Public Declare Function usb_bulk_write Lib "libusb0" _
    (ByRef dev As usb_dev_handle, ByVal ep As Long, ByVal pbytes As Long, ByVal size As Long, ByVal timeout As Long) As Long
Public Declare Function usb_bulk_read Lib "libusb0" _
    (ByRef dev As usb_dev_handle, ByVal ep As Long, ByVal pbytes As Long, ByVal size As Long, ByVal timeout As Long) As Long
Public Declare Function usb_interrupt_write Lib "libusb0" _
    (ByRef dev As usb_dev_handle, ByVal ep As Long, ByVal pbytes As Long, ByVal size As Long, ByVal timeout As Long) As Long
Public Declare Function usb_interrupt_read Lib "libusb0" _
    (ByRef dev As usb_dev_handle, ByVal ep As Long, ByVal pbytes As Long, ByVal size As Long, ByVal timeout As Long) As Long
Public Declare Function usb_control_msg Lib "libusb0" _
    (ByRef dev As usb_dev_handle, ByVal requesttype As Long, ByVal request As Long, ByVal value As Long, ByVal Index As Long, _
                                                    ByVal pbytes As Long, ByVal size As Long, ByVal timeout As Long) As Long
Public Declare Function usb_set_configuration Lib "libusb0" _
    (ByRef dev As usb_dev_handle, ByVal configuration As Long) As Long
Public Declare Function usb_claim_interface Lib "libusb0" _
    (ByRef dev As usb_dev_handle, ByVal interface As Long) As Long
Public Declare Function usb_release_interface Lib "libusb0" _
    (ByRef dev As usb_dev_handle, ByVal interface As Long) As Long
Public Declare Function usb_set_altinterface Lib "libusb0" _
    (ByRef dev As usb_dev_handle, ByVal alternate As Long) As Long
Public Declare Function usb_resetep Lib "libusb0" _
    (ByRef dev As usb_dev_handle, ByVal ep As Long) As Long 'unsigned int ep
Public Declare Function usb_clear_halt Lib "libusb0" _
    (ByRef dev As usb_dev_handle, ByVal ep As Long) As Long 'unsigned int ep
Public Declare Function usb_reset Lib "libusb0" _
    (ByRef dev As usb_dev_handle) As Long

'Alle Subs und Functions di nichts auf dem Stack bergeben/bernehmen laufen sowohl als CDecl auch als StdCall
Public Declare Function usb_strerror Lib "libusb0" () As Long  ' char *
Public Declare Sub usb_init Lib "libusb0" ()
Public Declare Sub usb_set_debug Lib "libusb0" (ByVal plevel As Long)
  
Public Declare Function usb_find_busses Lib "libusb0" () As Long
Public Declare Function usb_find_devices Lib "libusb0" () As Long
Public Declare Function usb_device Lib "libusb0" _
    (ByRef dev As usb_dev_handle) As Long   'usb_device *
Public Declare Function usb_get_busses Lib "libusb0" () As Long ' usb_bus *

'  /* Windows specific functions */
Public Declare Sub usb_install_service_np Lib "libusb0" ()
Public Declare Sub usb_uninstall_service_np Lib "libusb0" ()
Public Declare Function usb_install_driver_np Lib "libusb0" (ByVal pinf_file As String) As Long
Public Declare Function usb_touch_inf_file_np Lib "libusb0" (ByVal pinf_file As String) As Long
Public Declare Function usb_install_needs_restart_np Lib "libusb0" () As Long
Public Declare Function usb_get_version Lib "libusb0" () As Long 'usb_version * 'delivers a pointer to a structure

Public Declare Function usb_isochronous_setup_async Lib "libusb0" _
    (ByRef dev As usb_dev_handle, ByVal ppcontext As Long, ByVal ep As Byte, ByVal pktsize As Long) As Long

Public Declare Function usb_bulk_setup_async Lib "libusb0" _
    (ByRef dev As usb_dev_handle, ByVal ppcontext As Long, ByVal ep As Byte) As Long
Public Declare Function usb_interrupt_setup_async Lib "libusb0" _
    (ByRef dev As usb_dev_handle, ByVal ppcontext As Long, ByVal ep As Byte) As Long

Public Declare Function usb_submit_async Lib "libusb0" _
    (ByVal pcontext As Long, ByVal pbytes As Long, ByVal size As Long) As Long
Public Declare Function usb_reap_async Lib "libusb0" _
    (ByVal pcontext As Long, ByVal timeout As Long) As Long
Public Declare Function usb_reap_async_nocancel Lib "libusb0" _
    (ByVal pcontext As Long, ByVal timeout As Long) As Long
Public Declare Function usb_cancel_async Lib "libusb0" _
    (ByVal pcontext As Long) As Long
Public Declare Function usb_free_async Lib "libusb0" _
    (ByVal ppcontext As Long) As Long

Public LibUSB0Dll As New CDeclDllFunctionCall

